module GandiCli
  class Cli < Thor
    desc "list", "List zone files"
    def list
      puts "%-30s %10s %s" % %w[name zone_id current_version]
      GandiCli::Zone.list(client).each do |zone|
        puts "%-30s %10s %15s" % [zone.name, zone.id, zone.version.id]
      end
    end

    desc "add_record ZONE_NAME TYPE TTL NAME VALUE", "Add record to zone"
    def add_record(zone_name, type, ttl, name, value)
      zone = GandiCli::Zone.find(client, zone_name)
      version = editable_version(zone)
      version.add_record(type, ttl.to_i, name, value)
    end

    desc "records ZONE_NAME VERSION", "List records for a given version of a zone"
    def records(zone_name, version)
      zone = GandiCli::Zone.find(client, zone_name)
      zone.find_version(version).records.each do |record|
        puts "%s %s %s %s %s" % [record.id, record.type, record.ttl, record.name, record.value]
      end
    end

    desc "activate ZONE_NAME VERSION", "Activate version for zone"
    def activate(zone_name, version)
      zone = GandiCli::Zone.find(client, zone_name)
      zone.find_version(version).activate!
    end

    desc "delete_record ZONE_NAME VERSION RECORD_ID", "Delete a record"
    def delete_record(zone_name, version, record_id)
      zone = GandiCli::Zone.find(client, zone_name)
      zone_record = zone.find_version(version).records.find{ |record|
        record.id == record_id.to_i
      }
      if zone_record
        puts "Deleting #{format_record(zone_record)}"
        zone_record.delete
      else
        puts "Record not found"
      end
    end

    desc "versions ZONE_NAME", "List versions for zone"
    def versions(zone_name)
      zone = GandiCli::Zone.find(client, zone_name)
      zone.versions.each do |version|
        puts "#{zone_name} v#{version.id}"
      end
    end

    desc "copy_version ZONE_NAME [VERSION]", "Copy zone version (defaults to active version)"
    def copy_version(zone_name, version = 0)
      zone = GandiCli::Zone.find(client, zone_name)
      new_version = zone.find_version(version.to_i).copy
      puts "New version is #{new_version.id}"
    end

    private

    def editable_version(zone)
      if zone.version.id == zone.versions.last.id
        puts "Creating new version of zone file based on v#{zone.version.id}"
        zone.version.copy
      else
        zone.versions.last
      end
    end

    def format_record(record)
      "%s %s %s %s %s" % [record.id, record.type, record.ttl, record.name, record.value]
    end

    def client
      GandiCli::Client.new(key).session
    end

    def zone_by_name(zone_name)
      GandiCli::Zone.find(client, zone_name)
    end

    def key
      File.read(ENV['HOME']+"/.gandi_cli").strip
    end
  end
end
